 package org.apache.jyve.util;

/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import org.apache.turbine.util.*;
import org.apache.turbine.services.localization.Localization;
import org.apache.turbine.services.resources.TurbineResources;
import org.apache.turbine.om.security.*;

/**
 * Utility for doing permission checks in Jyve Screens and Actions.
 * 
 * Sample usage: <br>
 * <code>
 * JSecurityCheck mycheck = new JSecurityCheck(data, "Unauthorized to do this!", "WrongPermission");
 * if ( !mycheck.visitorHasPermission("add_topic");
 *     return;
 * </code>
 * 
 * @author <a href="mailto:bhoeneis@ee.ethz.ch"> Bernie H&ouml;neisen</a>, ETH Zurich
 */
public class JSecurityCheck
{
    
    private String message;
    private String failScreen;
    private RunData data = null;
    
    /**
     *  Constructor.
     *  @param message message to display upon failure
     *  @param failScreen screen to redirect to upon failure.
     */    
    public JSecurityCheck ( RunData data, String message, String failScreen )
    {
        this.data=data;
        this.message=message;
        this.failScreen=failScreen;
    }
    
    /**
     *  Easy constructor. You should NOT use this constructor in combination
     *  with methods that use data.setMessage(message) or setSreen(failScreen). 
     *  Rather use the constructor:
     *  public JSecurityCheck ( RunData data, String message, String failScreen )
     *  in these cases.
     */    
    public JSecurityCheck ( RunData data )
    {
        this.data=data;
        this.message="Programming Error: The wrong constructor has been used in the JSecurityCheck class!";
        this.failScreen=TurbineResources.getString("screen.error");
    }

    /**
     *  This method checks whether a user (=visitor) has access
     *  to the New<i>Tablename</i> screen.
     *  @param tablename project, faq, topic, question or answer
     *  @return true if visitor has access, false if visitor has no access  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean accessToScreenNew(String tablename) throws Exception
    {
        return hasAddPerm(tablename) ;
    }
    
    /**
     *  This method checks whether a user (=visitor) has access
     *  to the AddNew<i>Tablename</i> screen.
     *  @param tablename project, faq, topic, question or answer
     *  @return true if visitor has access, false if visitor has no access  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean accessToActionAddNew(String tablename) throws Exception
    {
        return hasAddPerm(tablename) ;
    }

    /**
     *  This method checks whether a user (=visitor) has access
     *  to the Edit<i>Tablename</i> screen.
     *  @param tablename project, faq, topic, question or answer
     *  @param idValue id of the selected item in <i>tablename</i>
     *  @return true if visitor has access, false if visitor has no access  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean accessToScreenEdit(String tablename, Integer idValue) throws Exception
    {
        return hasModOrModOwnPerm(tablename, idValue) ;
    }
    
   /**
     *  This method checks whether a user (=visitor) has access
     *  to the Update<i>Tablename</i> action.
     *  @param tablename project, faq, topic, question or answer
     *  @param idValue id of the selected item in <i>tablename</i>
     *  @return true if visitor has access, false if visitor has no access  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean accessToActionUpdate(String tablename, Integer idValue) throws Exception
    {
        return hasModOrModOwnPerm(tablename, idValue) ;
    }

   /**
     *  This method checks whether a user (=visitor) has access
     *  to the Release<i>Tablename</i> action.
     *  @param tablename project, faq, topic, question or answer
     *  @return true if visitor has access, false if visitor has no access  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean accessToActionRelease(String tablename) throws Exception
    {
        return hasReleasePerm(tablename) ;
    }

    /** 
     *  This method checks whether a user (=visitor) has a permission
     *  to add a released <i>tablename</i>.
     *  @param tablename project, faq, topic, question or answer
     *  @return true if visitor has a permission, false if visitor has no permission  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean permissionToAddReleased(String tablename) throws Exception
    {
        if ( visitorHasPermission("add_" + tablename))
        {
            return true;
        }
        return false;
    }
    
    /** 
     *  This method checks whether a user (=visitor) has the permission
     *  to see the unreleased entries of <i>tablename</i>.
     *  @param tablename project, faq, topic, question or answer
     *  @return true if visitor has a permission, false if visitor has no permission  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean permissionToSeeUnreleased(String tablename) throws Exception
    {
        if ( visitorHasPermission("see_unreleased_" + tablename))
        {
            return true;
        }
        return false;
    }
    /** 
     *  This method checks whether a user (=visitor) has the permission
     *  to release entries of <i>tablename</i>.
     *  @param tablename project, faq, topic, question or answer
     *  @return true if visitor has a permission, false if visitor has no permission  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean permissionToRelease(String tablename) throws Exception
    {
        if ( visitorHasPermission("release_" + tablename))
        {
            return true;
        }
        return false;
    }

    /**
     *  This method checks whether a user (=visitor) has
     *  the add_<i>tablename</i> or the add_unreleased_<i>tablename</i>
     *  permission. If not, it sets message and failscreen.
     *  @param tablename project, faq, topic, question or answer
     *  @return true, if visitor has the permission; false, if visitor has not the permission  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean hasAddPerm(String tablename) throws Exception
    {        
        if ( visitorHasPermission("add_" + tablename))
        {
            return true;
        }
        if ( visitorHasPermission("add_unreleased_" + tablename))
        {
            return true;
        }
        data.setScreen(failScreen);
        data.setMessage(message);
        return false;
    }

    /**
     *  This method checks whether a user (=visitor) has the modify_<i>tablename</i>
     *  or modify_own_<i>tablename</i> or modify_own_unreleased_<i>tablename</i>
     *  permission for a given id. If not, it sets message and failscreen.
     *  @param tablename project, faq, topic, question or answer
     *  @param idValue id of the selected item in <i>tablename</i>
     *  @return true, if visitor has the permission; false, if visitor has not the permission  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean hasModOrModOwnPerm(String tablename, Integer idValue) throws Exception
    {        
        if ( visitorHasPermission("modify_" + tablename))
        {
            return true;
        }
        
        // get visitor ID of Record and visitor ID of the current user (=visitor)
        RecordExtractor recEx = new RecordExtractor (data);
        int visitorIdofRecord = recEx.getVisitorId ( tablename, idValue );
        int visitorId = data.getUser().getId();

        // check for valid range
        if ( ( visitorId <= 0 ) || ( visitorIdofRecord <= 0 ) )
        {
            data.setScreen(failScreen);
            if (visitorId <= 0 )
            {
                data.setMessage(Localization.getString("JSECURITYCHECK_RANGEVISITORID") + new Integer(visitorId).toString() );
            }
            else
            {
                data.setMessage(Localization.getString("JSECURITYCHECK_RANGEVISITORIDREC") + new Integer(visitorIdofRecord).toString() );
            }
            return false; 
        }

        // check for permission modify_own_<tablename>
        if ( ( visitorIdofRecord == visitorId ) && ( visitorHasPermission("modify_own_" + tablename) ) )
        {
            return true;
        }
        
        // check for permission modify_own_unreleased_<tablename>
        if (    ( visitorIdofRecord == visitorId )
             && ( visitorHasPermission("modify_own_unreleased_" + tablename) )
             && ( !recEx.isReleased(tablename, idValue) ) )
        {
            return true;
        }
        data.setScreen(failScreen);
        data.setMessage(message);
        return false;
    }
    
    /**
     *  This method checks whether a user (=visitor) has
     *  the release_<i>tablename</i>. If not, it sets message and failscreen.
     *  @param tablename project, faq, topic, question or answer
     *  @return true, if visitor has the permission; false, if visitor has not the permission  
     *  @author Bernie Hoeneisen, ETH Zurich, mailto:bhoeneis@ee.ethz.ch
     */
    public boolean hasReleasePerm(String tablename) throws Exception
    {        
        if ( visitorHasPermission("release_" + tablename) )
        {
            return true;
        }
        data.setScreen(failScreen);
        data.setMessage(message);
        return false;
    }

    /**
     *  Does the user (=visitor) have this role?
     *  @param role role to be checked
     *  @return true, if visitor has the role; false, if visitor has not the role       
     */
    public boolean visitorHasRole(Role role) throws Exception
    {
        if ( data.getACL() == null || !data.getACL().hasRole(role) )
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    /**
     *  Does the user (=visitor) have this permission
     *  @param permission permission to be checked
     *  @return true, if visitor has the permission; false, if visitor has not the permission  
     */
    public boolean visitorHasPermission(String permission) throws Exception
    {
        if ( data.getACL() == null || !data.getACL().hasPermission(permission) )
        {
            return false;
        }
        else
        {
            return true;
        }
    }
        
    /**
     *  Get the message that should be displayed. This is initialized
     *  in the constructor.
     *  @return message that should be displayed
     */
    public String getMessage()
    {
        return message;
    }
    
    /**
     *  Get the screen that should be displayed. This is initialized
     *  in the constructor.
     *  @return screen that should be displayed
     */
    public String getFailScreen()
    {
        return failScreen;
    }    
}
